defmodule TuskerPush.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  @impl true
  def start(_type, _args) do
    OpentelemetryBandit.setup()
    OpentelemetryPhoenix.setup(adapter: :bandit)

    children = [
      TuskerPushWeb.Telemetry,
      {DNSCluster, query: Application.get_env(:tusker_push, :dns_cluster_query) || :ignore},
      {Phoenix.PubSub, name: TuskerPush.PubSub},
      # Start a worker by calling: TuskerPush.Worker.start_link(arg)
      # {TuskerPush.Worker, arg},
      {Finch,
       name: TuskerPush.Finch,
       pools: %{
         ("https://" <> TuskerPush.Apns.host(:development)) => [protocols: [:http2], count: 1],
         ("https://" <> TuskerPush.Apns.host(:production)) => [protocols: [:http2], count: 1]
       }},
      TuskerPush.Apns.Token,
      # Start to serve requests, typically the last entry
      TuskerPushWeb.Endpoint
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: TuskerPush.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  @impl true
  def config_change(changed, _new, removed) do
    TuskerPushWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
